/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern QuestionsGenerator, NaturalParameter, Subtraction */

/**
 * @param: operation: Constructor of Question for integer operation.
 */
function IntegerOperationsGenerator(operation) {
    // Call base constructor
    this.base = QuestionsGenerator;
    this.base();

    this.parameters = {
        table: new NaturalParameter("table", "Table de :"),
        lowerOperand: new NaturalParameter("lowerOperand", "Opérande inférieur :"),
        upperOperand: new NaturalParameter("upperOperand", "Opérande supérieur :")
    };
    this.operation = operation;
}

IntegerOperationsGenerator.prototype = new QuestionsGenerator();
IntegerOperationsGenerator.prototype.constructor = IntegerOperationsGenerator;


function IntegerOperationsGenerator_getMaxDifferentQuestions() {
    if (!this.allParametersSet()) {
        throw "Some parameters have not been set.";
    }

    var result = (this.parameters.upperOperand.value - this.parameters.lowerOperand.value + 1);
    if (this.operation.twoWays) {
        result = (result * 2) - 1; // 3 x 3 asked only once.
    }

    return result;
}

IntegerOperationsGenerator.prototype.getMaxDifferentQuestions = IntegerOperationsGenerator_getMaxDifferentQuestions;


function IntegerOperationsGenerator_generateNewQuestion() {
    var table = null;
    var lowerOperand = null;
    var upperOperand = null;
    var operandFirst = null;
    var randomOperand = null;
    var question = null;

    if (!this.allParametersSet()) {
        throw "Some parameters have not been set.";
    }

    table = this.parameters.table.value;
    lowerOperand = this.parameters.lowerOperand.value;
    upperOperand = this.parameters.upperOperand.value;

    if (this.operation.twoWays) {
        operandFirst = Math.round(Math.random());
    }
    else {
        operandFirst = false;
    }
    randomOperand = Math.floor((Math.random() * (upperOperand - lowerOperand + 1)) + lowerOperand);
    if (operandFirst) {
        question = new this.operation(randomOperand, table);
    }
    else {
        question = new this.operation(table, randomOperand);            
    }
    return question;
}

IntegerOperationsGenerator.prototype.generateNewQuestion = IntegerOperationsGenerator_generateNewQuestion;

