/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

function Question() {
    this.answeredValue = null;
    this.askDate = null;
    this.answerDate = null;
}

/**
 * Abstract: must be overridden.
 */
function Question_getStatement() {
}

Question.prototype.getStatement = Question_getStatement;

function Question_ask() {
    this.askDate = new Date();
    return this.getStatement();
}

Question.prototype.ask = Question_ask;

/**
 * Abstract: must be overridden.
 */
function Question_getResult() {
}

Question.prototype.getResult = Question_getResult;

function Question_answer(value) {
    this.answeredValue = value;
    this.answerDate = new Date();
}

Question.prototype.answer = Question_answer;

/**
 * Abstract: must be overridden.
 */
function Question_isCorrect() {
}

Question.prototype.isCorrect = Question_isCorrect;

function Question_getDuration() {
    return (this.answerDate.getTime() - this.askDate.getTime());
}

Question.prototype.getDuration = Question_getDuration;

/**
 * Abstract: must be overridden.
 */
function Question_equals() {
}

Question.prototype.equals = Question_equals;
