/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern Tester, Multiplication, Division, Addition, Subtraction, IntegerOperationsGenerator, $ */

var tester = null;

var actionTimeout = null;
var actionToDoAfterTimeout = function() {}; // Skip

function clearAnswer() {
    var element = document.getElementById("answer");
    element.value = "";
    element = document.getElementById("correctionDiv");
    element.className = "invisible";
}

function setCounter(id, value) {
    var counterSpan = document.getElementById(id);
    counterSpan.firstChild.data = value;
}

function clearCounters() {
    setCounter("correct", 0);
    setCounter("total", 0);
    setCounter("remainingCount", 0);
    setCounter("incorrectCount", 0);
    setCounter("correctCount", 0);
}

function showParameters() {
    document.getElementById("message").className = "hidden";
    document.getElementById("allParams").className = "frame";
    document.getElementById("testDiv").className = "hidden";
    document.getElementById("resultDiv").className = "hidden";
}

function showTest() {
    document.getElementById("message").className = "hidden";
    document.getElementById("allParams").className = "hidden";
    document.getElementById("testDiv").className = "frame";
    document.getElementById("resultDiv").className = "hidden";
}

function showResult() {
    document.getElementById("message").className = "hidden";
    document.getElementById("allParams").className = "hidden";
    document.getElementById("testDiv").className = "hidden";
    document.getElementById("resultDiv").className = "frame";
}

function showMessage(title, text) {
    var element;

    element = document.getElementById("messageTitle");
    element.firstChild.data = title;
    element = document.getElementById("messageText");
    element.firstChild.data = text;

    document.getElementById("message").className = "frame";
    document.getElementById("allParams").className = "hidden";
    document.getElementById("testDiv").className = "hidden";
    document.getElementById("resultDiv").className = "hidden";
}

function doAfterTimeout(funct, milliseconds) {
    if (milliseconds === 0) {
        funct();
    }
    else {
        actionToDoAfterTimeout = funct;
        actionTimeout = setTimeout(funct, milliseconds);
    }
}

function hideTestMessage() {
    document.getElementById("testMessage").className = "hidden";
}

function restartButton_Click() {
    showParameters();
}

function askNextQuestion() {
    clearAnswer();

    var element = document.getElementById("question");
    element.firstChild.data = tester.nextQuestion();
    
    $("#remainingCountDiv").effect("transfer",
                                   { to: "#question" },
                                   1000,
                                   function () {				   
                                       setCounter("remainingCount", tester.getRemainingCount());
                                   });

    element = document.getElementById("answer");
    element.disabled = false;
    element.focus();
}

function body_Load() {
    showMessage("Bienvenue", "Teste tes connaissances ou exerce-toi pour faire des progrès !");
    doAfterTimeout(showParameters, 5000);
}

function continueButton_Click() {
    clearTimeout(actionTimeout);
    actionToDoAfterTimeout();
}

function startTest() {
    showTest();

    clearCounters();
    document.getElementById("remainingCount").firstChild.data =
        tester.getRemainingCount();

    askNextQuestion();
}

function startButton_Click() {
    var title = "";
    var questionsGenerator = null;

    showTest();

    var tableTypeElement = document.getElementById("multiplicationOption");
    if (tableTypeElement.checked) {
        questionsGenerator = new IntegerOperationsGenerator(Multiplication);
        title = "Multiplications : table de ";
    }
    else {
        tableTypeElement = document.getElementById("additionOption");
        if (tableTypeElement.checked) {
            questionsGenerator = new IntegerOperationsGenerator(Addition);
            title = "Additions : table de ";
        }
        else {
            tableTypeElement = document.getElementById("divisionOption");
            if (tableTypeElement.checked) {
                questionsGenerator = new IntegerOperationsGenerator(Division);
                title = "Divisions : table de ";
            }
            else {
                questionsGenerator = new IntegerOperationsGenerator(Subtraction);
                title = "Soustractions : table de ";
            }
        }
    }

    var table = document.getElementById("table").value;
    questionsGenerator.parameters.table.setValue(table);
    var operand = document.getElementById("lowerOperand").value;
    questionsGenerator.parameters.lowerOperand.setValue(operand);
    operand = document.getElementById("upperOperand").value;
    questionsGenerator.parameters.upperOperand.setValue(operand);

    title += table;
    document.getElementById("testTitle").firstChild.data = title;

    tester = new Tester(questionsGenerator);

    showMessage(title, "Voyons quel est ton niveau !");
    doAfterTimeout(startTest, 5000);
}

function fillCell(cell, value) {
    cell.firstChild.data = value;
}

function buildReview() {
    var rows;
    var row;
    var question;
    var duration;
    var maxDuration;

    maxDuration = document.getElementById("max").value;

    rows = document.getElementsByTagName("tr");
    // First row is in table header: skip it.
    for (var i = 1; i < rows.length; i += 1) {
        row = rows[i];

        row.className = "hidden";
        if (tester.getTotalCount() < i) {
            // Nothing else to do: row just has to be hidden.
            continue;
        }

        question = tester.questions[i - 1];

        fillCell(row.childNodes[0], question.getStatement());
        fillCell(row.childNodes[1], question.answeredValue);
        if (question.isCorrect()) {
            fillCell(row.childNodes[2], "");
        }
        else {
            fillCell(row.childNodes[2], question.getResult());
        }

        duration = question.getDuration() / 1000;
        duration = (Math.round(duration * 10) / 10);
        fillCell(row.childNodes[3],
                 duration + " seconde" +
                 (duration > 1 ? "s" : ""));

        row.className = "";
        if (duration > maxDuration) {
            row.className = "late";
        }

        if (!question.isCorrect()) {
            row.className = "wrong";
        }
    }
}

function updateScore() {
    setCounter("correct", tester.getCorrectCount());
    setCounter("total", tester.getTotalCount());

    var score = document.getElementById("score");
    switch (tester.getIncorrectCount()) {
    case 0:
        score.className = "good";
        break;
    case 1:
        score.className = "bad";
        break;
    default:
        score.className = "ugly";
        break;
    }
}

function finishTest() {
    var incorrectCount = tester.getIncorrectCount();
    var testMode = document.getElementById("testModeOption").checked;
    if (testMode) {
        buildReview();
        updateScore();
        showMessage("Terminé", "Tu as fait " +
                    incorrectCount +
                    (incorrectCount < 2 ? " faute !" : " fautes !"));
        doAfterTimeout(showResult, 5000);
    }
    else {  // Training mode
        if (incorrectCount === 0) {
            showMessage("Fin de l'entraînement", "Tu n'as pas fait de fautes.");
            doAfterTimeout(showParameters, 5000);
        }
        else {
            tester.resetForTraining();
            showMessage("Entraînement", "Reprenons là où tu t'es trompé !");
            doAfterTimeout(startTest, 5000);
        }
    }
}

function keyPressHandler(event) {
    var answer = document.getElementById("answer");
    var yesNo = document.getElementById("yesNo");
    var correctionDiv = document.getElementById("correctionDiv");
    var correctAnswer = document.getElementById("correctAnswer");
    var correctCount;
    var incorrectCount;

    if (event.keyCode === 13) { // Enter key pressed.
        answer.disabled = true;

        if (tester.answerCurrent(answer.value)) {
            yesNo.firstChild.data = "Oui !";
            correctionDiv.className = "correct";
            correctAnswer.className = "hidden";
            correctAnswer.firstChild.data = "";
            correctCount = tester.getCorrectCount();
            $("#answer").effect("transfer",
                                { to: "#correctCountDiv" },
                                1000,
                                function () {
                                    setCounter("correctCount", correctCount);
                                });
        }
        else {
            yesNo.firstChild.data = "Non !";
            correctionDiv.className = "wrong";
            correctAnswer.className = "";
            correctAnswer.firstChild.data =
                tester.getCurrentQuestionCorrection();
            incorrectCount = tester.getIncorrectCount();
            $("#answer").effect("transfer",
                                { to: "#incorrectCountDiv" },
                                1000,
                                function () {
                                    setCounter("incorrectCount", incorrectCount);
                                });
        }
        
        var delay = document.getElementById("delay").value;
        delay = delay - 0;
        if (tester.lastReached()) {
            doAfterTimeout(finishTest, delay * 1000);
        }
        else {
            doAfterTimeout(askNextQuestion, delay * 1000);
        }

        return false;
    }
}
