/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

function Tester(questionsGenerator) {
    /**
     * Array of Question objects for current test.
     */
    this.questions = null;

    /**
     * Index in this.questions.
     */
    this.current = -1;

    /**
     * Array of Question objects with an incorrect answer.
     */
    this.errors = null;


    // Constructor code
    var numberOfQuestions = questionsGenerator.getMaxDifferentQuestions();
    this.questions = questionsGenerator.generateQuestions(numberOfQuestions);
    this.errors = [];
    this.current = -1;
}

function Tester_getTotalCount() {
    return this.questions.length;
}

Tester.prototype.getTotalCount = Tester_getTotalCount;

function Tester_getAlreadyAskedCount() {
    return this.current + 1;
}

Tester.prototype.getAlreadyAskedCount = Tester_getAlreadyAskedCount;

function Tester_getRemainingCount() {
    return (this.getTotalCount() - this.getAlreadyAskedCount());
}

Tester.prototype.getRemainingCount = Tester_getRemainingCount;

function Tester_getIncorrectCount() {
    return this.errors.length;
}

Tester.prototype.getIncorrectCount = Tester_getIncorrectCount;

function Tester_getCorrectCount() {
    return (this.getAlreadyAskedCount() - this.getIncorrectCount());
}

Tester.prototype.getCorrectCount = Tester_getCorrectCount;

function Tester_answerCurrent(value) {
    var currentQuestion = this.questions[this.current];
    currentQuestion.answer(value);
    if (currentQuestion.isCorrect()) {
        return true;
    }
    else {
        this.errors.push(currentQuestion);
    }
}

Tester.prototype.answerCurrent = Tester_answerCurrent;

function Tester_nextQuestion() {
    this.current += 1;
    return this.questions[this.current].ask();
}

Tester.prototype.nextQuestion = Tester_nextQuestion;

function Tester_lastReached() {
    return (this.getRemainingCount() === 0);
}

Tester.prototype.lastReached = Tester_lastReached;

function Tester_getCurrentQuestionCorrection() {
    var currentQuestion = this.questions[this.current];
    return currentQuestion.getStatement() + currentQuestion.getResult();
}

Tester.prototype.getCurrentQuestionCorrection = Tester_getCurrentQuestionCorrection;

function Tester_resetForTraining() {
    this.questions = this.errors;
    this.errors = [];
    this.current = -1;
}

Tester.prototype.resetForTraining = Tester_resetForTraining;
