/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern Addition, Question,
  assert, assertEquals, assertNotEquals, assertNotUndefined, assertFalse */

/**
 * @returns The names of all test functions in this page.
 */
function exposeTestFunctionNames() {
    return [
        "testCorrectInheritanceFromQuestion",
        "testGetResultSimple",
        "testGetStatement",
        "testIsCorrectSimple",
        "testIsCorrectEmptyAnswer",
        "testEquals"
    ];
}

function testCorrectInheritanceFromQuestion() {
    var addi = new Addition(1, 1);
    var quest = new Question();
    
    assertNotUndefined("ask method", addi.ask);
    assertEquals("Addition should not override ask method.",
		 quest.ask, addi.ask);
    assertNotUndefined("answer method", addi.answer);
    assertEquals("Addition should not override answer method.",
		 quest.answer, addi.answer);
    assertNotUndefined("getDuration method", addi.getDuration);
    assertEquals("Addition should not override getDuration method.",
		 quest.getDuration, addi.getDuration);

    assertNotUndefined("getStatement method", addi.getStatement);
    assertNotEquals("Addition should override getStatement method.",
		    addi.getStatement, quest.getStatement);
    assertNotUndefined("getResult method", addi.getResult);
    assertNotEquals("Addition should override getResult method.",
		    addi.getResult, quest.getResult);
    assertNotUndefined("isCorrect method", addi.isCorrect);
    assertNotEquals("Addition should override isCorrect method.",
                    addi.isCorrect, quest.isCorrect);
    assertNotUndefined("equals method", addi.equals);
    assertNotEquals("Addition should override equals method.",
		    addi.equals, quest.equals);
}

function testGetStatement() {
    var addition = new Addition(7, 6);
    assertEquals("7 + 6 = ", addition.getStatement());
}

function testGetResultSimple() {
    var addition = new Addition(2, 3);
    assertEquals(5, addition.getResult());

    addition = new Addition(10, 7);
    assertEquals(17, addition.getResult());
}

function testIsCorrectSimple() {
    var addition = new Addition(7, 8);
    addition.answer("15");
    assert("String value convertible into correct answer should be correct.",
           addition.isCorrect());
    addition.answer("a");
    assertFalse("String value not convertible into correct answer should not be correct.",
                addition.isCorrect());
}

function testIsCorrectEmptyAnswer() {
    var addition = new Addition(0, 0);
    assertFalse("Should be incorrect before answered.", addition.isCorrect());
    addition.answer("");
    assertFalse("Empty answer should be incorrect.", addition.isCorrect());
    addition.answer("0");
    assert("Zero should sometimes be correct.", addition.isCorrect());
}

function testEquals() {
    var addi1 = new Addition(3, 4);
    var addi2 = new Addition(3, 4);
    assert("Two Addition objects with same operands in same order should be equal.",
           addi1.equals(addi2));

    addi1 = new Addition(3, 3);
    addi2 = new Addition(3, 3);
    assert("Two doubles with same operands should be equal.",
           addi1.equals(addi2));
}
