/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern Multiplication, Question,
  assert, assertEquals, assertNotEquals, assertNotUndefined, assertFalse */

/**
 * @returns The names of all test functions in this page.
 */
function exposeTestFunctionNames() {
    return [
        "testCorrectInheritanceFromQuestion",
        "testGetResultSimple",
        "testGetStatement",
        "testIsCorrectSimple",
        "testIsCorrectEmptyAnswer",
        "testEquals"
    ];
}

function testCorrectInheritanceFromQuestion() {
    var mult = new Multiplication(1, 1);
    var quest = new Question();
    
    assertNotUndefined("ask method", mult.ask);
    assertEquals("Multiplication should not override ask method.",
		 quest.ask, mult.ask);
    assertNotUndefined("answer method", mult.answer);
    assertEquals("Multiplication should not override answer method.",
		 quest.answer, mult.answer);
    assertNotUndefined("getDuration method", mult.getDuration);
    assertEquals("Multiplication should not override getDuration method.",
		 quest.getDuration, mult.getDuration);

    assertNotUndefined("getStatement method", mult.getStatement);
    assertNotEquals("Multiplication should override getStatement method.",
		    mult.getStatement, quest.getStatement);
    assertNotUndefined("getResult method", mult.getResult);
    assertNotEquals("Multiplication should override getResult method.",
		    mult.getResult, quest.getResult);
    assertNotUndefined("isCorrect method", mult.isCorrect);
    assertNotEquals("Multiplication should override isCorrect method.",
                    mult.isCorrect, quest.isCorrect);
    assertNotUndefined("equals method", mult.equals);
    assertNotEquals("Multiplication should override equals method.",
		    mult.equals, quest.equals);
}

function testGetStatement() {
    var multiplication = new Multiplication(7, 6);
    assertEquals("7 x 6 = ", multiplication.getStatement());
}

function testGetResultSimple() {
    var multiplication = new Multiplication(2, 3);
    assertEquals(6, multiplication.getResult());

    multiplication = new Multiplication(10, 7);
    assertEquals(70, multiplication.getResult());
}

function testIsCorrectSimple() {
    var multiplication = new Multiplication(7, 8);
    multiplication.answer("56");
    assert("String value convertible into correct answer should be correct.",
           multiplication.isCorrect());
    multiplication.answer("a");
    assertFalse("String value not convertible into correct answer should not be correct.",
                multiplication.isCorrect());
}

function testIsCorrectEmptyAnswer() {
    var multiplication = new Multiplication(1, 0);
    assertFalse("Should be incorrect before answered.", multiplication.isCorrect());
    multiplication.answer("");
    assertFalse("Empty answer should be incorrect.", multiplication.isCorrect());
    multiplication.answer("0");
    assert("Zero should sometimes be correct.", multiplication.isCorrect());
}

function testEquals() {
    var mult1 = new Multiplication(3, 4);
    var mult2 = new Multiplication(3, 4);
    assert("Two Multiplication objects with same operands in same order should be equal.",
           mult1.equals(mult2));

    mult1 = new Multiplication(3, 3);
    mult2 = new Multiplication(3, 3);
    assert("Two squares with same operands should be equal.",
           mult1.equals(mult2));
}
